#ifndef LLOYDSFORGMM_H
#define LLOYDSFORGMM_H

#include "../randomalgorithm.h"

#include "../../base/gmmdesc.h"
#include "../../base/commonutil.h"

/**
* @brief Lloyd type algorithm for GMMs
*/
class LloydsForGMM : public RandomAlgorithm
{
public:
	/**
	* @brief Constructor initializing input and configuring the algorithm.
	* @remarks Remember to init the mixture model.
	*/
	LloydsForGMM(commonutil::DataSet const& ds, bool v, uint32_t seed, fp_type ratio = -1., bool spherical = false);
	
	LloydsForGMM(commonutil::DataSet const& ds, bool v, std::mt19937& gen, fp_type ratio = -1., bool spherical = false);

	virtual ~LloydsForGMM()
	{
	}

	virtual void init(unsigned int);
	virtual void init(GMMDesc const& gmmdesc);
	virtual void run(unsigned int numSteps = 1);

	/**
	* @brief does a dynamic cast of the given GMMalgorithm to LloydsForGMM
	* @return NULL if the Algorithm is not a LloydsForGMM instance
	*/
	static LloydsForGMM* toLloydsForGMM(GMMAlgorithm* a);

private:
	fp_type ratio;
	bool spherical;
};

#endif
