#ifndef BASE_H
#define BASE_H

//#define _GLIBCXX_DEBUG
//#define GLFW_CDECL // has to be defined before #include <AntTweakBar.h> due to problems with Callbacks


#define NDEBUG

#include <cassert>
#include <cstddef>
#include <Eigen/Dense>
#include <random>
#include <stdint.h>

typedef double fp_type;
typedef Eigen::MatrixXd Matrix;
typedef Eigen::VectorXd Vector;
typedef Eigen::DenseIndex idx_type;

enum GMMLabMode { DEFAULT, DIFF, LOOKAT, RNGTEST, CLASSIFY };
enum TestLabMode { DEFAULT_TEST, DATA };

enum GenMethod
{
	NORMAL_META_MEAN_AND_COVAR,
	UNIFORM_MEANS_AND_RESTRICTED_COVAR,
	DIFFERENT_SHAPES,
};

enum MeasurementError{ NO_ME, BLUR_CORRELATED, BLUR_DIFFICULT, UNIFORM_NOISE, GAUSSIAN_NOISE };

enum AlgorithmID 
{ 
  NONE, 
  EM
};
  
enum InitMethod 
{ 
  EMPTY, 
  
  // GMM-intializations
  // ... kmeans initializations "2gmm"
  UNIFORM_MEANS2GMM, 
  UNIFORM_LLOYDS_MEANS2GMM,
  ADAPTIVE_LLOYDS_MEANS2GMM,
  GONZALEZ_LLOYDS_MEANS2GMM,
  ADAPTIVE_MEANS2GMM, 
  GONZALEZ2GMM,
  // ... sequential algorithms
  ALTERNATELY_ADAPTIVEMEANS_MEANS2GMM, 
  ALTERNATELY_ADAPTIVEMEANS_MEANS2GMM_THEN_LLOYDSFORGMM,
  ADAPTIVE_GAUSSIANS,
  GONZALEZ_FORGMM,
  GONZALEZ_FORGMM_THEN_LLOYDSFORGMM,
  GONZALEZ_KWEDLO,
  // .. agglomerative
  SAMPLE_AVGLINK_MEANS2GMM,
    
};

enum AntiPropMode { INVERSEDENSITY, MAHALANOBIS };

enum PropMode { DENSITY, NEIGHBORHOOD };

enum CostMeasure{ KMEANS, NLL, FUZZY};


#ifndef M_PI
#define M_PI           3.14159265358979323846
#endif

#define gmmlab_throw(message) \
{printf("ERROR: %s\n     in file %s at line %d\n", message,__FILE__,__LINE__); throw(1);}

#endif
